//+------------------------------------------------------------------+
//|                                                          Complex |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "2009, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "2.00"

#define _EURUSD	0
#define _GBPUSD	1
#define _EURGBP	2
#define _EURJPY	3
#define _USDJPY	4
#define _GBPJPY	5

#property indicator_separate_window
#property indicator_buffers 50
#property indicator_plots   1

#property indicator_label1 "Complex2"
#property indicator_type1 DRAW_LINE
#property indicator_color1 clrGreen
#property indicator_style1 STYLE_SOLID
#property indicator_width1 1

//--- input parameters
input int MA_Slow=56;
input ENUM_TIMEFRAMES MA_Slow_TF=PERIOD_H1;
input int MA_Fast=9;
input ENUM_TIMEFRAMES MA_Fast_TF=PERIOD_H1;
input int nBars=500;
input double kUSD=1;
input double kGBP=1;
input double kJPY=1;
input double kEUR=1;

//------------------------------------------------------------------	class CSeries
class CSeries
{
public:
	double open[], high[], low[], close[]; //      
	int ho, hh, hl, hc; //  

public:
	CSeries() { ho=INVALID_HANDLE; hh=INVALID_HANDLE; hl=INVALID_HANDLE; hc=INVALID_HANDLE; };
	~CSeries() { IndicatorRelease(ho); IndicatorRelease(hh); IndicatorRelease(hl); IndicatorRelease(hc); }
public:
	void SetIndexBuf(int istart); //   
	void CreateHandle(string name, ENUM_TIMEFRAMES TF, int MA); //  
	int CopyBuf(int n); //   
};
//------------------------------------------------------------------	SetIndexBuf
void CSeries::SetIndexBuf(int istart) //   
{
	SetIndexBuffer(istart+0, open,INDICATOR_CALCULATIONS);
	SetIndexBuffer(istart+1, high, INDICATOR_CALCULATIONS);
	SetIndexBuffer(istart+2, low, INDICATOR_CALCULATIONS);
	SetIndexBuffer(istart+3, close, INDICATOR_CALCULATIONS);
	ArraySetAsSeries(open, true);
	ArraySetAsSeries(high, true);
	ArraySetAsSeries(low, true);
	ArraySetAsSeries(close, true);
}
//------------------------------------------------------------------	CreateHandle
void CSeries::CreateHandle(string name, ENUM_TIMEFRAMES TF, int MA) //  
{
	ho=iMA(name, TF, MA, 0, MODE_LWMA, PRICE_OPEN);
	hh=iMA(name, TF, MA, 0, MODE_LWMA,PRICE_HIGH);
	hl=iMA(name, TF, MA, 0, MODE_LWMA, PRICE_LOW);
	hc=iMA(name, TF, MA, 0, MODE_LWMA, PRICE_CLOSE);
}
//------------------------------------------------------------------	CopyBuf
int CSeries::CopyBuf(int n) //   
{
	int c=0; //   ,      
	if ((c=CopyBuffer(ho, 0, 0, n, open))<n) return(-1);
	if ((c=CopyBuffer(hh, 0, 0, n, high))<n) return(-1);
	if ((c=CopyBuffer(hl, 0, 0, n, low))<n) return(-1);
	if ((c=CopyBuffer(hc, 0, 0, n, close))<n) return(-1);
	return(n);
}

//------------------------------------------------------------------	class CPair
class CPair
{
public:
	string m_name; //  
	CSeries F, S; // 
	void SetIndexBuf(int istart) { S.SetIndexBuf(istart); F.SetIndexBuf(istart+4); }
	void CreateHandle(ENUM_TIMEFRAMES sTF, int sMA, ENUM_TIMEFRAMES fTF, int fMA) { S.CreateHandle(m_name, sTF, sMA); F.CreateHandle(m_name, fTF, fMA); }
	int CopyBuf(int n) { if (S.CopyBuf(n)<0) return(-1); if (F.CopyBuf(n)<0) return(-1); return(n); }
public:
	CPair() { };
	~CPair() { };
public:
};

double Mediana[];
CPair Pair[6];

//------------------------------------------------------------------	OnInit
int OnInit()
{
	SetIndexBuffer(0, Mediana, INDICATOR_DATA);
	ArraySetAsSeries(Mediana, true);
	//   
	Pair[0].m_name="EURUSD"; Pair[1].m_name="GBPUSD"; Pair[2].m_name="EURGBP";
	Pair[3].m_name="EURJPY"; Pair[4].m_name="USDJPY"; Pair[5].m_name="GBPJPY";
	for (int i=0; i<6; i++) //     
	{
		Pair[i].SetIndexBuf((i+1)+8*i);
		Pair[i].CreateHandle(MA_Slow_TF, MA_Slow, MA_Fast_TF, MA_Fast);
	}   
	string shortname;
	StringConcatenate(shortname, "Complex(",MA_Slow,",",MA_Fast,")");
	PlotIndexSetString(0, PLOT_LABEL, shortname); //     DataWindow
	IndicatorSetString(INDICATOR_SHORTNAME, shortname); //          
	IndicatorSetInteger(INDICATOR_DIGITS, 5); //    
	return(0);
}
//------------------------------------------------------------------	OnCalculate
int OnCalculate(const int rates_total, const int prev_calculated,
                const datetime& time[], const double& open[], const double& high[], const double& low[], const double& close[],
                const long& tick_volume[], const long& volume[], const int& spread[])
{

	for (int i=0; i<6; i++) if (Pair[i].CopyBuf(nBars)<0) { Print("Wait Data for "+Pair[i].m_name); return(0); } //       //   
	
	int limit=rates_total-prev_calculated; if (limit>=rates_total) limit=rates_total-1; //   
	if (limit>nBars) limit=nBars; //     
	
	for(int i=0; i<limit; i++) //       
	{
		Mediana[i]=EMPTY_VALUE;
		
		double OPEN=
			 (Pair[_EURUSD].F.open[i]-Pair[_EURUSD].S.open[i])*10000*kUSD
			+(Pair[_EURGBP].F.open[i]-Pair[_EURGBP].S.open[i])*10000*kGBP
			+(Pair[_EURJPY].F.open[i]-Pair[_EURJPY].S.open[i])*100*kJPY
			-(Pair[_EURUSD].S.open[i]-Pair[_EURUSD].F.open[i])*10000*kEUR
			+(Pair[_GBPUSD].S.open[i]-Pair[_GBPUSD].F.open[i])*10000*kGBP
			+(Pair[_USDJPY].F.open[i]-Pair[_USDJPY].S.open[i])*100*kJPY;
		
		double HIGH=
			 (Pair[_EURUSD].F.high[i]-Pair[_EURUSD].S.high[i])*10000*kUSD
			+(Pair[_EURGBP].F.high[i]-Pair[_EURGBP].S.high[i])*10000*kGBP
			+(Pair[_EURJPY].F.high[i]-Pair[_EURJPY].S.high[i])*100*kJPY
			-(Pair[_EURUSD].S.high[i]-Pair[_EURUSD].F.high[i])*10000*kEUR
			+(Pair[_GBPUSD].S.high[i]-Pair[_GBPUSD].F.high[i])*10000*kGBP
			+(Pair[_USDJPY].F.high[i]-Pair[_USDJPY].S.high[i])*100*kJPY;
		
		double LOW=
			 (Pair[_EURUSD].F.low[i]-Pair[_EURUSD].S.low[i])*10000*kUSD
			+(Pair[_EURGBP].F.low[i]-Pair[_EURGBP].S.low[i])*10000*kGBP
			+(Pair[_EURJPY].F.low[i]-Pair[_EURJPY].S.low[i])*100*kJPY
			-(Pair[_EURUSD].S.low[i]-Pair[_EURUSD].F.low[i])*10000*kEUR
			+(Pair[_GBPUSD].S.low[i]-Pair[_GBPUSD].F.low[i])*10000*kGBP
			+(Pair[_USDJPY].F.low[i]-Pair[_USDJPY].S.low[i])*100*kJPY;
		
		double CLOSE=
			 (Pair[_EURUSD].F.close[i]-Pair[_EURUSD].S.close[i])*10000*kUSD
			+(Pair[_EURGBP].F.close[i]-Pair[_EURGBP].S.close[i])*10000*kGBP
			+(Pair[_EURJPY].F.close[i]-Pair[_EURJPY].S.close[i])*100*kJPY
			-(Pair[_EURUSD].S.close[i]-Pair[_EURUSD].F.close[i])*10000*kEUR
			+(Pair[_GBPUSD].S.close[i]-Pair[_GBPUSD].F.close[i])*10000*kGBP
			+(Pair[_USDJPY].F.close[i]-Pair[_USDJPY].S.close[i])*100*kJPY;

		Mediana[i]=(OPEN+HIGH+LOW+CLOSE)/4;
	}
	return(rates_total);
}
